package com.ruoyi.logistics.controller;

import java.util.List;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

import com.ruoyi.common.security.utils.SecurityUtils;
import com.ruoyi.logistics.constant.ImportTemplateConstants;
import com.ruoyi.logistics.util.ExcelImportValidato;
import com.ruoyi.system.api.domain.SysUser;
import com.ruoyi.system.domain.dto.DeptImportDTO;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.log.annotation.Log;
import com.ruoyi.common.log.enums.BusinessType;
import com.ruoyi.common.security.annotation.RequiresPermissions;
import com.ruoyi.logistics.domain.BizAddressBook;
import com.ruoyi.logistics.service.IBizAddressBookService;
import com.ruoyi.common.core.web.controller.BaseController;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.ruoyi.common.core.utils.poi.ExcelUtil;
import com.ruoyi.common.core.web.page.TableDataInfo;
import org.springframework.web.multipart.MultipartFile;

/**
 * PC-地址簿管理
 * 
 * @author RuiJing
 * @date 2026-01-29
 */
@RestController
@RequestMapping("/book")
public class BizAddressBookController extends BaseController
{
    @Autowired
    private IBizAddressBookService bizAddressBookService;

    /**
     * 查询地址簿管理列表
     */
    @RequiresPermissions("system:book:list")
    @GetMapping("/list")
    public TableDataInfo list(BizAddressBook bizAddressBook)
    {
        startPage();
        List<BizAddressBook> list = bizAddressBookService.selectBizAddressBookList(bizAddressBook);
        return getDataTable(list);
    }

    /**
     * 导出地址簿管理列表
     */
    @RequiresPermissions("system:book:export")
    @Log(title = "地址簿管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizAddressBook bizAddressBook)
    {
        List<BizAddressBook> list = bizAddressBookService.selectBizAddressBookList(bizAddressBook);
        ExcelUtil<BizAddressBook> util = new ExcelUtil<BizAddressBook>(BizAddressBook.class);
        util.exportExcel(response, list, "地址簿管理数据");
    }

    /**
     * 获取地址簿管理详细信息
     */
    @RequiresPermissions("system:book:query")
    @GetMapping(value = "/{addressId}")
    public AjaxResult getInfo(@PathVariable("addressId") Long addressId)
    {
        return success(bizAddressBookService.selectBizAddressBookByAddressId(addressId));
    }

    /**
     * 新增地址簿管理
     */
    @RequiresPermissions("system:book:add")
    @Log(title = "地址簿管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizAddressBook bizAddressBook)
    {
        return success(bizAddressBookService.insertBizAddressBook(bizAddressBook));
    }

    /**
     * 修改地址簿管理
     */
    @RequiresPermissions("system:book:edit")
    @Log(title = "地址簿管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizAddressBook bizAddressBook)
    {
        return toAjax(bizAddressBookService.updateBizAddressBook(bizAddressBook));
    }

    /**
     * 删除地址簿管理
     */
    @RequiresPermissions("system:book:remove")
    @Log(title = "地址簿管理", businessType = BusinessType.DELETE)
	@DeleteMapping("/{addressIds}")
    public AjaxResult remove(@PathVariable Long[] addressIds)
    {
        return toAjax(bizAddressBookService.deleteBizAddressBookByAddressIds(addressIds));
    }

    /**
     * 获取登录用户默认地址
     */
    @GetMapping("/getDefaultAddress")
    public AjaxResult getDefaultAddress() {
        return success(bizAddressBookService.selectDefaultAddressBook());
    }


    /**
     * 导入地址簿
     */
    @RequiresPermissions("system:book:add")
    @Log(title = "导入地址簿", businessType = BusinessType.IMPORT)
    @PostMapping("/importData")
    public AjaxResult importData(MultipartFile file) throws IOException {




        ExcelUtil<BizAddressBook> util = new ExcelUtil<>(BizAddressBook.class);
        List<BizAddressBook> bookList = util.importExcel(file.getInputStream());

        if (CollectionUtils.isEmpty(bookList)) {
            return AjaxResult.error("导入数据不能为空！");
        }
        // 标准校验：是否为规定模板
        boolean isTemplate = ExcelImportValidato.isValidTemplate(
                file,
                ImportTemplateConstants.ADDRESS_TEMPLATE_HEAD
        );

        if (!isTemplate) {
            return AjaxResult.error("请使用系统提供的导入模板，请勿修改表头！");
        }
        String message = bizAddressBookService.importBook(bookList);
        return success(message);
    }


    /**
     * 下载地址簿导入模板
     */
    @RequiresPermissions("system:dept:add")
    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response) throws IOException {
        ExcelUtil<BizAddressBook> util = new ExcelUtil<BizAddressBook>(BizAddressBook.class);
        util.importTemplateExcel(response, "地址簿数据");
    }
}
